@@ -729,7 +729,7 @@ DeliverStateNotifyEvent(DeviceIntPtr dev, WindowPtr win)
         evcount += ((nval - 3) + 6)/6;
     }
 
-    BUG_RETURN(evcount <= ARRAY_SIZE(sev));
+    BUG_RETURN(evcount > ARRAY_SIZE(sev));
 
     FixDeviceStateNotify(dev, ev, k, b, v, first);
 
@@ -738,7 +738,7 @@ DeliverStateNotifyEvent(DeviceIntPtr dev, WindowPtr win)
         (ev - 1)->deviceid |= MORE_EVENTS;
         bev->type = DeviceButtonStateNotify;
         bev->deviceid = dev->id;
-        memcpy((char *) &bev->buttons[4], (char *) &b->down[4],
+        memcpy((char *) &bev->buttons[0], (char *) &b->down[4],
                DOWN_LENGTH - 4);
     }
 
@@ -781,6 +781,7 @@ DeviceFocusEvent(DeviceIntPtr dev, int type, int mode, int detail,
     len = sizeof(xXIFocusInEvent) + btlen * 4;
 
     xi2event = calloc(1, len);
+    BUG_RETURN(xi2event == NULL);
     xi2event->type = GenericEvent;
     xi2event->extension = IReqCode;
     xi2event->evtype = type;
