@@ -176,7 +176,7 @@ xorg_backtrace(void)
 #define ElfSym Elf32_Sym
 #endif
 
-/* Called for each frame on the stack to print it's contents */
+/* Called for each frame on the stack to print its contents */
 static int
 xorg_backtrace_frame(uintptr_t pc, int signo, void *arg)
 {
@@ -227,6 +227,8 @@ xorg_backtrace_frame(uintptr_t pc, int signo, void *arg)
 #endif                          /* HAVE_WALKCONTEXT */
 
 #ifdef HAVE_PSTACK
+#include <unistd.h>
+
 static int
 xorg_backtrace_pstack(void)
 {
